
/*
 * hybridcon.c: utility for talking to Hybrid cable/microwave modems over UDP 
 *
 * Written by and (c) Adam Fritzler (mid@auk.cx)  [21 Aug 1999]
 *
 *
 * Change log...
 *
 *  21 Aug 1999: Initial release.
 * 
 */
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <arpa/inet.h>
#include <getopt.h>

#define HYBRIDPORT 7777

int sendcmd(int fd, char *cmd)
{
  int framelen;
  unsigned char *frame;
  unsigned char hdr[] = {
    0x00, 0x00, 0x01, 
    0x05, /* unknown use...sometimes 0x05, 0x17, or 0x21 */
    0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x01, 
    0xff, 0xff, 0xff, 0xff, 
    0x00, 0x00, 
    0x0f, 0x0a
  };
  
  framelen = 20 + 2 + 6 + strlen(cmd) + 1;
  frame = (unsigned char *)calloc(1, framelen);
  memcpy(frame, hdr, 20);
  frame[20] = (strlen(cmd)>>8) & 0xff;
  frame[21] = strlen(cmd) & 0xff;
  /* 22 - 27 assumed NULL */
  memcpy(frame+28, cmd, strlen(cmd));
  return send(fd, frame, framelen, 0);  
}

int docmd(int fd, char *cmd)
{
  unsigned char readbuf[1024];
  int n;
  if (sendcmd(fd, cmd)<0)
    {
      perror("sendcmd");
      exit(-1);
    }
  if ((n = recvfrom(fd, readbuf, 1024, 0, NULL, NULL))<0)
    {
      perror("recvfrom");
      exit(-1);
    }
  readbuf[n] = '\0';
  
  printf("\n%s\n", readbuf+18);
  return 0;
}

void usage(void)
{
  fprintf(stderr, "hybridcon: access the console of Hybrid cable modems, v1.0\n");
  fprintf(stderr, "usage: hybridcon <modem IP>\n");
  exit(-1);
}

int main (int argc, char **argv)
{
  int keepgoing = 1;
  int fd;
  struct sockaddr_in modemaddr;
  char inbuf[1024];
  int n = 0;
  char *modem;
  char *cmd = NULL;

  while((n = getopt(argc, argv, "c:h")) != EOF)
    {
      switch(n)
	{
	case 'c':
	  cmd = optarg;
	  break;
	case 'h':
	  usage();
	  break;
	default:
	  break;
	}
    }
  
  if (optind == argc)
    usage();
  else
    modem = argv[optind];

  bzero(&modemaddr, sizeof(modemaddr));
  modemaddr.sin_family = AF_INET;
  modemaddr.sin_port = htons(HYBRIDPORT);
  inet_pton(AF_INET, modem, &modemaddr.sin_addr);

  if ((fd = socket(AF_INET, SOCK_DGRAM, 0))<0)
    {
      perror("socket");
      exit(-1);
    }
  if (connect(fd, &modemaddr, sizeof(modemaddr))<0)
    {
      perror("connect");
      exit(-1);
    }

  if (cmd) /* given command on command line */
    {
      docmd(fd, cmd);
      exit (0);
    }

  docmd(fd, "version");
  
  while(keepgoing)
    {
      printf("[hybridcon] ");
      fgets(inbuf, 1024, stdin);
      if (strlen(inbuf))
	{
	  if (!strncmp("exit", inbuf, 4))
	    keepgoing = 0;
	  else
	    {
	      inbuf[strlen(inbuf)-1] = '\0';
	      if (strlen(inbuf))
		docmd(fd, inbuf);
	    }
	}
    }

  return 0;
}
